#
#   PROBE - Launch parameters screen
#

from pygame import draw, Rect
from albow.controls import Label, TextField
from albow.resource import get_font, get_image
from albow.dialogs import alert
from vector_algebra import add
from controls import NumericField, ImageButton
from screen import ProbeScreen

class LaunchParamsScreen(ProbeScreen):

	bg_image = get_image("desk-paper-a4-left-800x600.jpg")
	left_margin = 50
	top_margin = 50
	width = 350
	height = 400
	label_margin = left_margin + 20
	header_top = top_margin + 20
	fields_top = header_top + 100
	field_margin = label_margin + 140
	field_spacing = 30
	bg_color = (255, 255, 255)
	fg_color = (50, 50, 50)
	input_color = (0, 0, 0)
	input_font = get_font(12, "VeraMono.ttf")
	small_font = get_font(9, "Vera.ttf")

	def __init__(self, shell):
		ProbeScreen.__init__(self, shell)
		self.input_fields = []
		self.add_header()
		self.launch_dir_field = NumericField(6, 3, signed = True)
		self.burnout_vel_field = NumericField(signed = False, exp = True)
		self.fuel_field = Label(" " * 20)
		self.signature_field = TextField(50)
		self.signature_field.enter_action = self.approve
		self.add_input_field(0, "LAUNCH HEADING", self.launch_dir_field, "deg")
		self.add_input_field(1, "BURNOUT VELOCITY", self.burnout_vel_field, "km/s")
		self.add_field(2, "FUEL LOAD", self.fuel_field)
		self.add_input_field(3, "APPROVED", self.signature_field)
		r = self.signature_field.rect
		self.add_label(r.left, r.bottom + 8, "(Write initials here and press Enter)",
			font = self.small_font)
		mc_button = ImageButton("MISSION CONTROL", "mission_control_thumb.png",
			action = self.go_to_mission_control)
		mc_button.rect.bottomright = add(shell.rect.bottomright, (-20, -20))
		self.add(mc_button)
		lp_button = ImageButton("MISSION PLANNING", "drawing_board_thumb.png",
			action = self.go_to_mission_planning)
		lp_button.rect.bottomright = add(mc_button.rect.topright, (0, -10))
		self.add(lp_button)
	
	def add_header(self):
		self.add_label(self.label_margin, self.header_top,
			"VEHICLE LAUNCH SPECIFICATIONS")
	
	def add_label(self, x, y, text, font = None):
		label = Label(text)
		label.rect.topleft = (x, y)
		if font:
			label.font = font
		self.add(label)
	
	def add_input_field(self, i, caption, field, unit = None):
		self.input_fields.append(field)
		field.change_action = self.update
		self.add_field(i, caption, field, unit)
	
	def add_field(self, i, caption, field, unit = None):
		y = self.fields_top + i * self.field_spacing
		#label = Label(caption)
		#label.rect.topleft = (self.label_margin, y + 2)
		self.add_label(self.label_margin, y + 2, caption)
		field.rect.topleft = (self.field_margin, y)
		field.fg_color = self.input_color
		field.font = self.input_font
		#self.add(label)
		self.add(field)
		if unit:
			self.add_label(field.rect.right + 10, field.rect.top + 2, unit)
	
	def draw(self, surf):
		color = self.fg_color
		frame = surf.get_rect()
		surf.blit(self.bg_image, (0, 0))
		r = Rect(self.left_margin, self.top_margin, self.width, self.height)
		draw.rect(surf, color, r, 1)
		for field in self.input_fields:
			r = field.rect
			y = r.bottom + 2
			draw.line(surf, color, (r.left, y), (r.right, y))

	def update(self):
		text = ""
		try:
			fuel = self.calculate_fuel()
			text = "%.0f kg" % fuel
		except MissingParam:
			pass
		except FuelCalculationError:
			text = "ERROR"
		self.fuel_field.text = text
	
	def approve(self):
		try:
			self.check_signature()
			system = self.get_system_and_vehicle()
			dir = self.get_direction()
			fuel = self.calculate_fuel()
			system.vehicle.prepare_for_launch(system.home, dir, fuel)
			alert("Pad Operations reports vehicle is fuelled and ready for launch.",
				bg_color = (0, 200, 0))
			self.go_to_mission_control()
		except NoSignature:
			self.signature_field.focus()
			self.report_error("Initials of Project Director required for approval.")
		except NoVehicle:
			self.report_error("There is no vehicle to prepare for launch.")
		except NoDirection:
			self.launch_dir_field.focus()
			self.report_error("Incomplete specification: Launch heading missing.")
		except NoVelocity:
			self.burnout_vel_field.focus()
			self.report_error("Incomplete specification: Burnout velocity missing or zero.")
		except FuelCalculationError:
			self.burnout_vel_field.focus()
			self.report_error("Error occurred while calculating fuel load. The "
				"burnout velocity you requested may be unattainable with this vehicle.")
	
	def report_error(self, mess):
		alert(mess, bg_color = (200, 0, 0))
		self.signature_field.set_text("")
	
	def check_signature(self):
		if not self.signature_field.get_text():
			raise NoSignature
	
	def get_system_and_vehicle(self):
		system = self.game.current_system
		if not system:
			raise NoVehicle
		if not system.vehicle:
			raise NoVehicle
		return system
	
	def get_direction(self):
		val = self.launch_dir_field.get_value()
		if val is None:
			raise NoDirection
		return val
	
	def get_velocity(self):
		vel = self.burnout_vel_field.get_value()
		if not vel:
			raise NoVelocity
		return vel * 1e3
	
	def calculate_fuel(self):
		system = self.get_system_and_vehicle()
		vel = self.get_velocity()
		try:
			fuel = system.vehicle.calculate_launch_fuel(system.home, vel)
		except Exception, e:
			print "------------- Exception while calculating fuel ----------"
			print e
			raise FuelCalculationError
		if fuel < 0:
			raise FuelCalculationError
		return fuel

	def go_to_mission_planning(self):
		self.shell.go_to_mission_planning()

	def go_to_mission_control(self):
		self.shell.go_to_mission_control()


class MissingParam(StandardError):
	pass

class NoSignature(MissingParam):
	pass

class NoDirection(MissingParam):
	pass

class NoVehicle(MissingParam):
	pass

class NoVelocity(MissingParam):
	pass

class FuelCalculationError(StandardError):
	pass

