#
#   PROBE - Objects that move in space
#

from math import pi, sin, cos
from random import randint, uniform, normalvariate, expovariate
from constants import au
from vector_algebra import direction
from names import random_word
from rocket_science import gravity, escape_velocity
import substances
from substances import generate_composition, dump_composition

system_radius = 5 * au

class SpaceObject(object):
	#  name
	#  kind
	#  position
	
	position = (0.0, 0.0)
	
	def __init__(self, name):
		self.name = name

	def get_astronomical_data(self, data):
		data.append(("Name", self.name))
		data.append(("Kind", self.kind))

class CelestialObject(SpaceObject):
	#  mass
	#  radius
	#  orbital_distance
	
	has_atmospheric_composition = False
	has_surface_composition = False
	
	def __init__(self, name, radius, orbital_distance):
		SpaceObject.__init__(self, name)
		self.radius = radius
		self.orbital_distance = orbital_distance
		self.mass = self.mean_density * sphere_volume(radius)
	
	def get_astronomical_data(self, data):
		SpaceObject.get_astronomical_data(self, data)
		data.append(("Radius", "%.2e km" % (1e-3 * self.radius)))
		data.append(("Mass", "%.2e kg" % (self.mass)))
		data.append(("Orbit", "%.2e km" % (1e-3 * self.orbital_distance)))

	def dump(self):
		print "Name:", self.name
		print "Kind:", self.kind
		print "Orbit: %.3e km" % (self.orbital_distance * 1e-3)
		print "Radius: %.3e km" % (self.radius * 1e-3)
		print "Mass: %.3e kg" % self.mass


class Star(CelestialObject):
	kind = "Star"
	mean_density = 1.4

	def __init__(self, name):
		CelestialObject.__init__(self, name, random_star_radius(), 0.0)


class Planet(CelestialObject):
	#  temperature
	#  atmospheric_pressure

	def __init__(self, orbital_distance):
		name = random_planet_name()
		radius = self.generate_radius()
		CelestialObject.__init__(self, name, radius, orbital_distance)
		self.position = self.generate_position()
		self.temperature = self.generate_temperature()
		self.atmospheric_pressure = self.generate_atmospheric_pressure()
		self.generate_data()
	
	def surface_gravity(self):
		return gravity(self.mass, self.radius)
	
	def escape_velocity(self):
		return escape_velocity(self.mass, self.radius)

	def get_astronomical_data(self, data):
		CelestialObject.get_astronomical_data(self, data)
		ag = self.surface_gravity()
		ve = self.escape_velocity()
		data.append(("Azimuth", "%7.3f deg" % direction(self.position)))
		data.append(("Surf grav", "%.3f m/s^2" % ag))
		data.append(("Esc vel", "%.3e km/s" % (1e-3 * ve)))
#		data.append(("Avg temperature", "%.0f C" % self.temperature))
#		data.append(("Atmospheric pressure", "%.1f kPa" % self.atmospheric_pressure))

	def dump(self):
		CelestialObject.dump(self)
		print "Temperature: %.0f C" % self.temperature
		if self.atmospheric_pressure is not None:
			print "Atmospheric pressure: %.0f kPa" % self.atmospheric_pressure
		print "Position: (%.2e, %.2e)" % (self.position[0], self.position[1])
		dump_composition("Atmospheric composition", self.atmospheric_composition)
		dump_composition("Surface composition", self.surface_composition)

	def generate_radius(self):
		lo = self.min_radius
		hi = self.max_radius
		mean = 0.5 * (lo + hi)
		sd = (hi - lo) / 6
		return normalvariate(mean, sd)

	def generate_position(self):
		a = uniform(-pi, pi)
		r = self.orbital_distance
		return (r * cos(a), r * sin(a))

	def generate_temperature(self):
		return -173 + 700 * (1 - self.orbital_distance / system_radius) + uniform(-100, 100)

	def generate_data(self):
		self.atmospheric_composition = generate_composition(
			self.atmosphere_substances)
		self.surface_composition = generate_composition(
			self.surface_substances)


class GasGiantPlanet(Planet):
	kind = "Gas giant"
	mean_density = 1.3e3
	min_radius = 25e6
	max_radius = 250e6
	atmosphere_substances = substances.gas_giant_atmosphere_substances
	surface_substances = []
	has_atmospheric_composition = True
	has_surface_composition = False
	
	def __init__(self, orbital_distance):
		Planet.__init__(self, orbital_distance)
	
	def generate_atmospheric_pressure(self):
		return None


class RockyPlanet(Planet):
	kind = "Rocky planet"
	mean_density = 6.5e3
	min_radius = 2e6
	max_radius = 25e6
	mean_atmospheric_pressure = 100
	atmosphere_substances = substances.rocky_planet_atmosphere_substances
	surface_substances = substances.rocky_planet_surface_substances
	has_atmospheric_composition = True
	has_surface_composition = True

	def generate_atmospheric_pressure(self):
		if uniform(0, 1) < self.radius / 3e6:
			return expovariate(1.0 / self.mean_atmospheric_pressure)
		else:
			return 0


class TerrestrialPlanet(RockyPlanet):
	kind = "Earthlike planet"
	mean_density = 5497
	mean_radius = 6378000
	delta_radius = 0.1 * mean_radius
	min_radius = mean_radius - delta_radius
	max_radius = mean_radius + delta_radius
	
	atmosphere_substances = substances.terrestrial_planet_atmosphere_substances
	surface_substances = substances.terrestrial_planet_surface_substances

	def generate_temperature(self):
		return normalvariate(20, 5)

#-----------------------------------------------------------------------------

def sphere_volume(r):
	return 4 * pi * r * r * r / 3

def random_planet_name():
	#return "Fiddlesticks%d" % randint(1, 1000)
	n = randint(1, 3) + randint(1, 3) + randint(1, 3)
	return random_word(n)

def random_star_radius():
 return 7e9 * normalvariate(1.0, 0.2)

if __name__ == "__main__":
	rp = RockyPlanet(1e20)
	gg = GasGiantPlanet(1e20)
	tp = TerrestrialPlanet(1e20)
	rp.dump()
	gg.dump()
	tp.dump()
