#
#   PROBE - Vehicles
#

from objects import SpaceObject
from vector_algebra import add, multiply, rectangular
from rocket_science import delta_v, fuel_for_liftoff

#---------------------------------------------------------------------------

class VehicleStage(object):
	#  name         string
	#  empty_mass   float   kg
	#  Isp          float   Ns/kg
	#  burn_rate    float   kg/s
	#  fuel         float   kg
	#  burn_time    float   s
	
	name = "?"
	
	def __init__(self, empty_mass, Isp_sec, burn_rate, fuel = 0, burn_time = 0):
		self.empty_mass = empty_mass
		self.Isp = Isp_sec * 9.8
		self.burn_rate = burn_rate
		self.fuel = fuel
		self.burn_time = burn_time
	
	def get_mass(self):
		return self.empty_mass + self.fuel

#---------------------------------------------------------------------------

class Vehicle(SpaceObject):
	#  instruments  [Instrument]
	#  stages       [VehicleStage]   uppermost first
	#  mass         float            total mass (readonly, calculated)
	#  velocity     vector
	#  attitude     float            degrees
	#  state        enum
	#  nearest_to   CelestialObject
	
	#  states
	UNPREPPED = 0
	PRELAUNCH = 1
	FLYING = 2
	CRASHED = 3
	
	state = UNPREPPED
	velocity = (0.0, 0.0)
	attitude = 0.0
	#burning = False
	nearest_to = None
	orbital_distance = None
	radius = 0

	def __init__(self, name):
		SpaceObject.__init__(self, name)
		self.instruments = []
		self.stages = []
	
	def add_instrument(self, instrument):
		self.instruments.append(instrument)
	
	def add_stage(self, stage):
		self.stages.append(stage)
	
	def get_mass(self):
		total = 0.0
		for stage in self.stages:
			total += stage.get_mass()
		return total
	
	mass = property(get_mass)
	
	def prepare_for_launch(self, planet, heading, fuel):
		assert len(self.stages) > 1, "Preparing vehicle with no booster"
		assert self.state <= self.PRELAUNCH, "Re-preparing used vehicle"
		self.attitude = heading
		booster = self.stages[-1]
		booster.fuel = fuel
		booster.burn_time = fuel / booster.burn_rate
		self.position = add(planet.position,
			rectangular(planet.radius, self.attitude))
		self.state = self.PRELAUNCH
		g = planet.surface_gravity() ###
		b = booster.burn_rate ###
		print "Vehicle.prepare_for_launch:" ###
		print "Thrust required  = %.3e N" % (g * self.mass) ###
		print "Thrust available = %.3e N" % (booster.Isp * b) ###

	def timestep(self, game, dt):
		if self.state == self.FLYING:
			self.position = add(self.position, multiply(dt, self.velocity))
			#print "Vehicle.timestep:", dt ##
			#print "...position =", self.position ###
			stage = self.current_stage()
			tb = stage.burn_time
			if tb > 0.0:
				dt = min(dt, tb)
				m = self.get_mass()
				dm = min(stage.fuel, dt * stage.burn_rate)
				#print "...burning %.3f kg" % dm ###
				dv = rectangular(delta_v(stage.Isp, m, dm), self.attitude)
				#print "...dv =", dv ###
				stage.fuel -= dm
				#print "...fuel =", stage.fuel ###
				self.velocity = add(dv, self.velocity)
				#print "...velocity =", self.velocity ###
				stage.burn_time -= dt
				if stage.fuel <= 0:
					self.drop_stage()
	
	def get_burn_time(self):
		return self.current_stage().burn_time
	
	def current_stage(self):
		return self.stages[-1]
	
	def drop_stage(self):
		stages = self.stages
		if len(stages) > 1:
			print "...DROPPING STAGE" ###
			del stages[-1]
			#if len(stages) < 2:
			#	self.burning = False

	def name_stages(self):
		n = len(self.stages)
		for j in xrange(n - 1, -1, -1):
			if j > 0:
				name = "B%d" % (j + 1)
			else:
				name = "OB"
			self.stages[j].name = name

	def calculate_launch_fuel(self, planet, v):
		booster = self.stages[-1]
		b = booster.burn_rate
		m = self.mass - booster.fuel
		g = planet.surface_gravity()
		return fuel_for_liftoff(booster.Isp, b, m, g, v, 0.01)

	def manoeuvre(self, attitude, burn_time):
		if self.state == self.FLYING and len(self.stages) == 1:
			if attitude is not None:
				self.attitude = attitude
			if burn_time is not None:
				self.stages[0].burn_time = burn_time

	def make_measurements(self, game, planet, distance):
		#print "Vehicle.make_measurements:", planet, "%.3e" % distance ###
		for instrument in self.instruments:
			data = []
			instrument.measure(data, planet, distance)
			for datum in data:
				game.store_datum(planet, datum)
